package quimufu.colourful_portals;

import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.FluidFillable;
import net.minecraft.block.ShapeContext;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.fluid.Fluid;
import net.minecraft.fluid.FluidState;
import net.minecraft.item.ItemPlacementContext;
import net.minecraft.item.Items;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.state.StateManager;
import net.minecraft.state.property.EnumProperty;
import net.minecraft.state.property.Properties;
import net.minecraft.util.BlockRotation;
import net.minecraft.util.DyeColor;
import net.minecraft.util.math.*;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.util.shape.VoxelShapes;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;
import net.minecraft.world.WorldAccess;
import org.jetbrains.annotations.Nullable;

import static quimufu.colourful_portals.ColourfulPortalsMod.PORTAL_MANAGER;


public class PortalBlock extends Block implements FluidFillable {
    public static final EnumProperty<Direction.Axis> AXIS = Properties.AXIS;
    public static final EnumProperty<DyeColor> DYE_COLOR = EnumProperty.of("colour", DyeColor.class);

    public static final VoxelShape X_AABB = Block.createCuboidShape(
            7.0D,
            0.0D,
            0.0D,
            9.0D,
            16.0D,
            16.0D
    );
    public static final VoxelShape Y_AABB = Block.createCuboidShape(
            0.0D,
            7.0D,
            0.0D,
            16.0D,
            9.0D,
            16.0D
    );
    public static final VoxelShape Z_AABB = Block.createCuboidShape(
            0.0D,
            0.0D,
            7.0D,
            16.0D,
            16.0D,
            9.0D
    );

    public PortalBlock(Settings settings) {
        super(settings);
        this.setDefaultState(this.stateManager.getDefaultState().with(AXIS, Direction.Axis.X).with(DYE_COLOR, DyeColor.BLACK));
    }

    @Override
    public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos blockPos, ShapeContext shapeContext) {
        if (shapeContext.isHolding(Items.DEBUG_STICK)
            || shapeContext.isHolding(ColourfulPortalsMod.PORTAL_BLOCK_ITEM)
            || shapeContext.isHolding(ColourfulPortalsMod.PORTAL_FLUID_BUCKET_ITEM)) {
            return getShape(state);
        }
        return VoxelShapes.empty();
    }

    public VoxelShape getShape(BlockState state) {
        return switch (state.get(AXIS)) {
            case Z -> Z_AABB;
            case Y -> Y_AABB;
            default -> X_AABB;
        };
    }

    @Override
    public BlockState rotate(BlockState state, BlockRotation rotation) {
        return changeRotation(state, rotation);
    }

    public static BlockState changeRotation(BlockState state, BlockRotation rotation) {
        switch (rotation) {
            case COUNTERCLOCKWISE_90, CLOCKWISE_90 -> {
                switch (state.get(AXIS)) {
                    case X -> {
                        return state.with(AXIS, Direction.Axis.Z);
                    }
                    case Z -> {
                        return state.with(AXIS, Direction.Axis.X);
                    }
                }
            }
        }
        return state;
    }

    public BlockState getStateWith(DyeColor color, Direction.Axis axis) {
        return this.getDefaultState().with(AXIS, axis).with(DYE_COLOR, color);
    }

    @Override
    public BlockState getPlacementState(ItemPlacementContext ctx) {
        return this.getDefaultState().with(AXIS, ctx.getPlayerLookDirection().getAxis());
    }

    @Override
    protected void appendProperties(StateManager.Builder<Block, BlockState> builder) {
        builder.add(AXIS).add(DYE_COLOR);
    }

    @Override
    public boolean isTransparent(BlockState state, BlockView world, BlockPos pos) {
        return true;
    }

    @Override
    public boolean canFillWithFluid(@Nullable PlayerEntity player, BlockView world, BlockPos pos, BlockState state, Fluid fluid) {
        if (((World) world).isClient() && fluid == ColourfulPortalsMod.PORTAL_FLUID) {
            return true;
        }
        return fluid == ColourfulPortalsMod.PORTAL_FLUID
               && (world instanceof ServerWorld)
               && PORTAL_MANAGER.canExtend((ServerWorld) world, pos);
    }


    @Override
    public boolean tryFillWithFluid(WorldAccess world, BlockPos pos, BlockState state, FluidState fluidState) {
        if (world.isClient() && fluidState.isOf(ColourfulPortalsMod.PORTAL_FLUID)) {
            return true;
        }
        return fluidState.isOf(ColourfulPortalsMod.PORTAL_FLUID)
               && (world instanceof ServerWorld)
               && PORTAL_MANAGER.extend((ServerWorld) world, pos);
    }
}
