package quimufu.colourful_portals.client.particle;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.fabricmc.fabric.api.client.particle.v1.FabricSpriteProvider;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.option.ParticlesMode;
import net.minecraft.client.particle.ParticleFactory;
import net.minecraft.client.particle.ParticleTextureSheet;
import net.minecraft.client.particle.SpriteBillboardParticle;
import net.minecraft.client.world.ClientWorld;
import net.minecraft.particle.ParticleGroup;
import net.minecraft.particle.SimpleParticleType;
import org.joml.Random;

import java.util.EnumMap;
import java.util.Map;
import java.util.Optional;

import static net.minecraft.client.option.ParticlesMode.*;

@Environment(value = EnvType.CLIENT)
public class ColourfulAirSparkleParticle extends SpriteBillboardParticle {

    public static ParticleGroup COLOURFUL_AIR_GROUP = new ParticleGroup(0) {
        private static final Random rnd = new Random(747906141);
        private static final EnumMap<ParticlesMode, Integer> COUNTS = new EnumMap<>(
                Map.of(
                        ALL, 1 << 12,
                        DECREASED, 1 << 11,
                        MINIMAL, 1 << 8
                )
        );

        @Override
        public int getMaxCount() {
            Integer count = COUNTS
                    .getOrDefault(MinecraftClient.getInstance().options.getParticles().getValue(), 0);
            return count / 4 + rnd.nextInt((count * 3) / 4);
        }
    };

    private final FabricSpriteProvider spriteProvider;
    private final int variant;

    protected ColourfulAirSparkleParticle(ClientWorld clientWorld,
                                          double posX, double posY, double posZ,
                                          FabricSpriteProvider spriteProvider

    ) {
        super(clientWorld, posX, posY, posZ);
        this.spriteProvider = spriteProvider;
        int variants = spriteProvider.getSprites().size() / 2;
        variant = random.nextInt(variants);
        setSprite(spriteProvider.getSprites().get(variant * 2));
        setMaxAge(20 + random.nextInt(20) + random.nextInt(20));
        gravityStrength = 0;
    }

    @Override
    public void tick() {
        super.tick();
        if (age == maxAge / 2) {
            setSprite(spriteProvider.getSprites().get(variant * 2 + 1));
        }
    }

    @Override
    public Optional<ParticleGroup> getGroup() {
        return Optional.ofNullable(COLOURFUL_AIR_GROUP);
    }

    @Override
    public ParticleTextureSheet getType() {
        return ParticleTextureSheet.PARTICLE_SHEET_TRANSLUCENT;
    }

    public static ParticleFactory<SimpleParticleType> create(FabricSpriteProvider provider) {
        return (parameters, world, x, y, z, velocityX, velocityY, velocityZ) ->
                new ColourfulAirSparkleParticle(world, x, y, z, provider);
    }


}
