package quimufu.colourful_portals.client.rendering;

import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.client.texture.Animator;
import net.minecraft.client.texture.NativeImage;
import net.minecraft.client.texture.SpriteContents;
import org.jetbrains.annotations.Nullable;

import java.util.List;

/**
 * This code is mostly copies from minecraft source. It is owned by mojang.
 */
public class AlphaBlendingAnimator implements Animator {
    int frame;
    int currentTime;
    
    final SpriteContents spriteContents;
    final SpriteContents.Animation animation;
    @Nullable
    private final AlphaBlendingInterpolation interpolation;

    public AlphaBlendingAnimator(SpriteContents spriteContents, SpriteContents.Animation animation) {
        this.spriteContents = spriteContents;
        this.animation = animation;
        this.interpolation = new AlphaBlendingInterpolation();
    }

    @Override
    public void tick(int x, int y) {
        ++this.currentTime;
        SpriteContents.AnimationFrame animationFrame = this.animation.frames.get(this.frame);
        if (this.currentTime >= animationFrame.time) {
            int i = animationFrame.index;
            this.frame = (this.frame + 1) % this.animation.frames.size();
            this.currentTime = 0;
            int j = this.animation.frames.get((int)this.frame).index;
            if (i != j) {
                this.animation.upload(x, y, j);
            }
        } else if (this.interpolation != null) {
            if (!RenderSystem.isOnRenderThread()) {
                RenderSystem.recordRenderCall(() -> this.interpolation.apply(x, y, this));
            } else {
                this.interpolation.apply(x, y, this);
            }
        }
    }

    @Override
    public void close() {
        if (this.interpolation != null) {
            this.interpolation.close();
        }
    }

    public class AlphaBlendingInterpolation
            implements AutoCloseable {
        private final NativeImage[] images;

        AlphaBlendingInterpolation() {
            this.images = new NativeImage[spriteContents.mipmapLevelsImages.length];
            for (int i = 0; i < this.images.length; ++i) {
                int j = spriteContents.getWidth() >> i;
                int k = spriteContents.getHeight() >> i;
                this.images[i] = new NativeImage(j, k, false);
            }
        }

        void apply(int x, int y, AlphaBlendingAnimator animator) {
            SpriteContents.Animation animation = animator.animation;
            List<SpriteContents.AnimationFrame> list = animation.frames;
            SpriteContents.AnimationFrame animationFrame = list.get(animator.frame);
            double d = 1.0 - (double)animator.currentTime / (double)animationFrame.time;
            int i = animationFrame.index;
            int j = list.get((animator.frame + 1) % list.size()).index;
            if (i != j) {
                for (int k = 0; k < this.images.length; ++k) {
                    int l = spriteContents.getWidth() >> k;
                    int m = spriteContents.getHeight() >> k;
                    for (int n = 0; n < m; ++n) {
                        for (int o = 0; o < l; ++o) {
                            int p = this.getPixelColor(animation, i, k, o, n);
                            int q = this.getPixelColor(animation, j, k, o, n);
                            int a = this.lerp(d, p >> 24 & 0xFF, q >> 24 & 0xFF);
                            int r = this.lerp(d, p >> 16 & 0xFF, q >> 16 & 0xFF);
                            int s = this.lerp(d, p >> 8 & 0xFF, q >> 8 & 0xFF);
                            int t = this.lerp(d, p & 0xFF, q & 0xFF);
                            this.images[k].setColor(o, n, a << 24 | r << 16 | s << 8 | t);
                        }
                    }
                }
                spriteContents.upload(x, y, 0, 0, this.images);
            }
        }

        private int getPixelColor(SpriteContents.Animation animation, int frameIndex, int layer, int x, int y) {
            return spriteContents.mipmapLevelsImages[layer].getColor(x + (animation.getFrameX(frameIndex) * spriteContents.getWidth() >> layer), y + (animation.getFrameY(frameIndex) * spriteContents.getHeight() >> layer));
        }

        private int lerp(double delta, int to, int from) {
            return (int)(delta * (double)to + (1.0 - delta) * (double)from);
        }

        @Override
        public void close() {
            for (NativeImage nativeImage : this.images) {
                nativeImage.close();
            }
        }
    }
}
