package quimufu.colourful_portals.entity;

import net.minecraft.client.world.ClientWorld;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityType;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.projectile.thrown.ThrownItemEntity;
import net.minecraft.item.Item;
import net.minecraft.predicate.entity.EntityPredicates;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.sound.SoundCategory;
import net.minecraft.util.hit.HitResult;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;
import quimufu.colourful_portals.ColourfulPortalsMod;
import quimufu.colourful_portals.util.TeleportParticleHelper;
import quimufu.colourful_portals.util.TeleportHelper;

import java.util.List;

public class ColourfulPearlEntity
        extends ThrownItemEntity {

    public ColourfulPearlEntity(EntityType<? extends ColourfulPearlEntity> entityType, World world) {
        super(entityType, world);
    }

    public ColourfulPearlEntity(World world, LivingEntity owner) {
        super(ColourfulPortalsMod.COLOURFUL_PEARL_ENTITY_TYPE, owner, world);
    }


    @Override
    protected Item getDefaultItem() {
        return ColourfulPortalsMod.COLOURFUL_PEARL_ITEM;
    }

    @Override
    public void onRemoved() {
        if (this.getRemovalReason() == RemovalReason.DISCARDED) {
            if (getWorld() instanceof ClientWorld clientWorld) {
                clientWorld.playSound(
                        this.getX(), this.getY(), this.getZ(),
                        ColourfulPortalsMod.TELEPORT_AWAY_SOUND,
                        SoundCategory.BLOCKS,
                        100F, 1.25F,
                        false);
            }

            List<Entity> entities = getWorld()
                    .getOtherEntities(this, this.getBoundingBox().expand(5));

            entities.stream()
                    .filter(e -> e.squaredDistanceTo(this) < 25)
                    .forEach(entity -> TeleportParticleHelper.spawnTeleportationParticles(entity, getWorld()));
        }
    }

    @Override
    protected void onCollision(HitResult hitResult) {
        if (!(getWorld() instanceof ServerWorld serverWorld)) {
            return;
        }
        if (isRemoved()) {
            return;
        }
        List<Entity> entities = getWorld()
                .getOtherEntities(this, this.getBoundingBox().expand(5), EntityPredicates.EXCEPT_SPECTATOR
                        .and(entity -> entity instanceof LivingEntity)
                        .and(entity -> !entity.getType().isIn(ColourfulPortalsMod.COLOURFUL_PEARL_NOT_TELEPORTABLE)));

        entities.stream()
                .filter(e -> e.squaredDistanceTo(this) < 25)
                .forEach(entity -> {
                    Vec3d pos = this.getPos();
                    TeleportHelper.markForTeleport(entity,pos.toVector3f(), serverWorld);
                });
        this.discard();
    }

}
