package quimufu.colourful_portals.item;

import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemUsage;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.sound.SoundCategory;
import net.minecraft.sound.SoundEvent;
import net.minecraft.sound.SoundEvents;
import net.minecraft.stat.Stats;
import net.minecraft.util.Hand;
import net.minecraft.util.TypedActionResult;
import net.minecraft.util.UseAction;
import net.minecraft.world.World;
import net.minecraft.world.event.GameEvent;
import quimufu.colourful_portals.entity.ColourfulPearlEntity;
import quimufu.colourful_portals.util.TeleportParticleHelper;
import quimufu.colourful_portals.util.TeleportHelper;

public class ColourfulPearlItem
        extends Item {
    private static final int MAX_USE_TIME = 5;

    public ColourfulPearlItem(Item.Settings settings) {
        super(settings);
    }

    @Override
    public ItemStack finishUsing(ItemStack stack, World world, LivingEntity user) {
        if(user instanceof PlayerEntity player){
            player.getItemCooldownManager().set(this, 50);
        }
        if (world instanceof ServerWorld serverWorld) {
            TeleportHelper.markForTeleport(user, user.getPos().toVector3f(), serverWorld);
        }
        stack.decrementUnlessCreative(1, user);
        user.emitGameEvent(GameEvent.EAT);
        return stack;
    }

    @Override
    public UseAction getUseAction(ItemStack stack) {
        return UseAction.DRINK;
    }

    @Override
    public SoundEvent getDrinkSound() {
        //todo
        return SoundEvents.ITEM_HONEY_BOTTLE_DRINK;
    }

    @Override
    public SoundEvent getEatSound() {
        //todo
        return SoundEvents.ITEM_HONEY_BOTTLE_DRINK;
    }

    @Override
    public int getMaxUseTime(ItemStack stack, LivingEntity user) {
        return MAX_USE_TIME;
    }

    @Override
    public TypedActionResult<ItemStack> use(World world, PlayerEntity user, Hand hand) {
        if (user.isSneaking()) {
            TeleportParticleHelper.spawnTeleportationParticles(user, world);
            return ItemUsage.consumeHeldItem(world, user, hand);
        }

        ItemStack stackInHand = user.getStackInHand(hand);
        world.playSound(null, user.getX(), user.getY(), user.getZ(), SoundEvents.ENTITY_ENDER_PEARL_THROW, SoundCategory.NEUTRAL, 0.5f, 0.4f / (world.getRandom().nextFloat() * 0.4f + 0.8f));
        user.getItemCooldownManager().set(this, 50);
        if (!world.isClient) {
            ColourfulPearlEntity colourfulPearlEntity = new ColourfulPearlEntity(world, user);
            colourfulPearlEntity.setItem(stackInHand);
            colourfulPearlEntity.setVelocity(user, user.getPitch(), user.getYaw(), 0.0f, 1.5f, 1.0f);
            world.spawnEntity(colourfulPearlEntity);
        }
        user.incrementStat(Stats.USED.getOrCreateStat(this));
        stackInHand.decrementUnlessCreative(1, user);
        return TypedActionResult.success(stackInHand, world.isClient());
    }
}
