package quimufu.colourful_portals.util;

import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.math.Vec3d;

import java.util.LinkedList;
import java.util.List;

public class RaycastHelper {


    public static List<BlockPos> passedBlocks(Vec3d from, Vec3d to) {
        LinkedList<BlockPos> psitions = new LinkedList<>();
        Vec3d difference = to.subtract(from);
        if(to.equals(from)){
            return List.of();
        }

        double[] diff = new double[]{difference.getX(), difference.getY(), difference.getZ()};
        double[] start = new double[]{from.getX(), from.getY(), from.getZ()};
        double[] pos = new double[]{from.getX(), from.getY(), from.getZ()};
        BlockPos currentBlockPos = BlockPos.ofFloored(pos[0], pos[1], pos[2]);

        double currDelta = 0;

        while (currDelta < 1) {
            psitions.add(currentBlockPos);
            double minMissing = 1;
            for (int i = 0; i < 3; i++) {
                if (diff[i] == 0) {
                    continue;
                }
                double curr;
                if ((curr = ((round(pos[i], Math.signum(diff[i])) - pos[i]) / diff[i])) < minMissing) {
                    minMissing = curr;
                }
            }
            currDelta += minMissing;

            for (int i = 0; i < 3; i++) {
                pos[i] = start[i] + currDelta * diff[i];
            }
            currentBlockPos = BlockPos.ofFloored(pos[0], pos[1], pos[2]);
        }
        BlockPos last = BlockPos.ofFloored(to);
        if(!psitions.getLast().equals(last)){
            psitions.add(last);
        }

        return psitions;

    }

    private static double round(double val, double signum) {
        if (signum > 0) {
            return Math.floor(val + signum);
        }
        return Math.ceil(val + signum);
    }

    public static boolean passedOnAxis(Vec3d from, Vec3d to, BlockPos blockPos, Direction.Axis axis) {
        double startAA = from.getComponentAlongAxis(axis);
        double centerAA = blockPos.toCenterPos().getComponentAlongAxis(axis);
        Vec3d diff = to.subtract(from);
        double differenceAA = diff.getComponentAlongAxis(axis);

        double hitAfter = (centerAA - startAA) / differenceAA;

        if (differenceAA == 0 || hitAfter > 1 || hitAfter < 0) {
            return false;
        }

        return BlockPos.ofFloored(from.add(diff.multiply(hitAfter)))
                .equals(blockPos);
    }
}
